import laniopy as la
import tkinter
import tkinter.font
import argparse
import sys
import functools
from typing import List

WINDOW_WIDTH = 300
WINDOW_HEIGHT = 200


def callbackDo(lanio: la.Lanio, doNum: int, _):
    lanio.outPio(doNum, not lanio.getOut(doNum))


def updateDoDi(lanio: la.Lanio, root, canvas: tkinter.Canvas, doIds: List[int], diIds: List[int]):
    try:
        if lanio.getDoNum() > 0:
            for num, state in enumerate(lanio.getOutAll()):
                canvas.itemconfig(doIds[num], fill=(
                    'red' if state else 'black'))
        if lanio.getDiNum() > 0:
            for num, state in enumerate(lanio.inPioAll()):
                canvas.itemconfig(diIds[num], fill=(
                    'red' if state else 'black'))
    except la.LanioError:
        # Error handling
        pass

    root.after(
        500,
        functools.update_wrapper(
            functools.partial(
                updateDoDi, lanio, root, canvas, doIds, diIds), updateDoDi
        ))


def main(address, port):
    '''
     Connect to LANIO
    '''
    lanio = la.Lanio()
    if not lanio.connect(address, port, 1):
        return

    '''
     Initialize tkinter
    '''
    root = tkinter.Tk()
    root.title(f'LANIO ({address})')
    root.geometry(f'{WINDOW_WIDTH}x{WINDOW_HEIGHT}')

    '''
     Create tkinter canvas
    '''
    fontFamily = tkinter.font.nametofont('TkFixedFont').actual()['family']
    canvas = tkinter.Canvas(root, bg='#e6dfc5')
    canvas.pack(fill=tkinter.BOTH, expand=True)
    # LANIO description
    descLabel = canvas.create_text(
        WINDOW_WIDTH - 10, WINDOW_HEIGHT - 45,
        text='LANIO Control Unit',
        anchor='se',
        fill='#65826d',
        font=tkinter.font.Font(root, family=fontFamily, size=10, weight='bold'))
    descPos = canvas.bbox(descLabel)
    # Model background color
    canvas.create_rectangle(
        descPos[0], descPos[3] + 5, WINDOW_WIDTH - 10, WINDOW_HEIGHT - 10,
        fill='#65826d',
        outline='#65826d',
        width=5)
    # Model name
    canvas.create_text(
        (descPos[2] + descPos[0]) // 2, (WINDOW_HEIGHT + descPos[3]) // 2,
        text=f'LA-{lanio.getModel().name[2:]}',
        fill='#e6dfc5',
        font=tkinter.font.Font(root, family=fontFamily, size=20, weight='bold'))
    # DO
    doLabel = canvas.create_text(
        WINDOW_WIDTH // 8,
        WINDOW_HEIGHT // 4,
        text='DO',
        fill='#65826d',
        font=tkinter.font.Font(root, family=fontFamily, size=15, weight='bold'))
    doPos = canvas.bbox(doLabel)
    canvas.create_line(
        doPos[0], doPos[3] + 2, WINDOW_WIDTH - 10, doPos[3] + 2,
        width=2,
        fill='#ddb44c')
    # DO state
    doIds = []
    for i in range(lanio.getDoNum()):
        x = doPos[2] + ((i + 1) * 20)
        y = doPos[1]
        canvas.create_text(
            x, y,
            text=f'{i + 1}',
            fill='#65826d',
            font=tkinter.font.Font(root, family=fontFamily, size=10, weight='bold'))
        # LANIO DO button
        y = y + 12
        outId = canvas.create_oval(
            x - 5, y - 5, x + 5, y + 5,
            fill='black',
            outline='')
        doIds.append(outId)
        canvas.tag_bind(outId, '<Button-1>',
                        functools.partial(callbackDo, lanio, i))
    # DI
    diLabel = canvas.create_text(
        WINDOW_WIDTH // 8,
        WINDOW_HEIGHT // 4 + 40,
        text='DI',
        fill='#65826d',
        font=tkinter.font.Font(root, family=fontFamily, size=15, weight='bold'))
    diPos = canvas.bbox(diLabel)
    canvas.create_line(
        diPos[0], diPos[3] + 2, WINDOW_WIDTH - 10, diPos[3] + 2,
        width=2,
        fill='#65826d')
    # DI state
    diIds = []
    for i in range(lanio.getDiNum()):
        x = diPos[2] + ((i + 1) * 20)
        y = diPos[1]
        canvas.create_text(
            x, y,
            text=f'{i + 1}',
            fill='#65826d',
            font=tkinter.font.Font(root, family=fontFamily, size=10, weight='bold'))
        # LANIO DI button
        y = y + 12
        diIds.append(canvas.create_oval(
            x - 5, y - 5, x + 5, y + 5,
            fill='black',
            outline=''))

    # overwrite labels
    if lanio.getAiNum() > 0 :
        canvas.itemconfig(diLabel, text='AI')

    root.after(
        500,
        functools.update_wrapper(
            functools.partial(
                updateDoDi, lanio, root, canvas, doIds, diIds), updateDoDi
        ))
    root.mainloop()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        prog=sys.argv[0], description='LANIO tkinter example'
    )
    parser.add_argument('-a', '--address', type=str, required=True)
    parser.add_argument('-p', '--port', type=int, required=True)
    args = parser.parse_args()
    main(args.address, args.port)
