
import enum


class LanioModel(enum.Enum):
    '''
        モデル
    '''
    NONE = -1
    # Digital
    LA2R3PP = 0    # LA-2R3P-P
    LA3R2P = 1     # LA-3R2P(-P)
    LA7P = 2       # LA-7P-A(-P)
    LA5R = 3       # LA-5R
    LA5T2S = 4     # LA-5T2S(-P)
    LA5PP = 5      # LA-5P-P
    LA3R3PP = 6    # LA-3R3P-P
    # Serial
    LA232R = 7     # LA-232R-P
    LA232T = 8     #
    LA485R = 9     # LA-485R-P
    LA485T = 10    # LA-485T
    # Digital (New)
    LA4R4PP = 17
    LA8R = 19      # LA-8R
    LA8T = 20      # LA-8T
    LA8PP = 21     #
    LA4T4SP = 22   # LA-4T4S-P
    LAN2R2P = 25   # LA-N2R2P-P
    # Analog
    LA2R3A = 40    # LA-2R3A
    LA2A3PP = 41   # LA-3A2P-P
    LA2R3AV2 = 42  # LA-2R3A(V2)
    LA3A2PP = 43   # LA-2A3P-P
    LA5AI = 44     # LA-5AI


class LanioDeviceType(enum.Enum):
    '''
        デバイス種別
    '''
    NONE = 0    # なし
    DIGITAL = 1  # 通常モデル
    WIFI = 2    # 無線モデル
    N_8P = 3    # LA-N/8点モデル
    ANALOG = 4  # アナログモデル


class LanioSpec:
    '''
        LANIO スペック
    '''

    def __init__(self,
                 doNum: int,
                 diNum: int,
                 hasAutoSw: LanioDeviceType,
                 hasPulseCount: LanioDeviceType,
                 hasTimeDisconnect: LanioDeviceType,
                 hasTimeNotice: LanioDeviceType,
                 hasTimeRetrans: LanioDeviceType,
                 hasTimeDiDetect: LanioDeviceType,
                 hasMailAlert: LanioDeviceType,
                 aoNum: int,
                 aiNum: int):
        self.doNum = doNum
        self.diNum = diNum
        self.hasAutoSw = hasAutoSw
        self.hasPulseCount = hasPulseCount
        self.hasTimeDisconnect = hasTimeDisconnect
        self.hasTimeNotice = hasTimeNotice
        self.hasTimeRetrans = hasTimeRetrans
        self.hasTimeDiDetect = hasTimeDiDetect
        self.hasMailAlert = hasMailAlert
        self.aoNum = aoNum
        self.aiNum = aiNum


LANIO_SPEC_DEFS = {
    LanioModel.NONE:
    LanioSpec(
        0,
        0,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA2R3PP:
    LanioSpec(
        2,
        3,
        LanioDeviceType.WIFI,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        0,
        0
    ),
    LanioModel.LA3R2P:
    LanioSpec(
        3,
        2,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        0,
        0
    ),
    LanioModel.LA7P:
    LanioSpec(
        0,
        7,
        LanioDeviceType.NONE,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        0,
        0
    ),
    LanioModel.LA5R:
    LanioSpec(
        5,
        0,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA5T2S:
    LanioSpec(
        5,
        2,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        LanioDeviceType.WIFI,
        0,
        0
    ),
    LanioModel.LA5PP:
    LanioSpec(
        0,
        5,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        0,
        0
    ),
    LanioModel.LA3R3PP:
    LanioSpec(
        3,
        3,
        LanioDeviceType.WIFI,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.DIGITAL,
        0,
        0
    ),
    LanioModel.LA232R:
    LanioSpec(
        2,
        2,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA232T:
    LanioSpec(
        2,
        2,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA485R:
    LanioSpec(
        2,
        2,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA485T:
    LanioSpec(
        2,
        2,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        LanioDeviceType.DIGITAL,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA4R4PP:
    LanioSpec(
        4,
        4,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        0,
        0
    ),
    LanioModel.LA8R:
    LanioSpec(
        8,
        0,
        LanioDeviceType.N_8P,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA8T:
    LanioSpec(
        8,
        0,
        LanioDeviceType.N_8P,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        0
    ),
    LanioModel.LA8PP:
    LanioSpec(
        0,
        8,
        LanioDeviceType.NONE,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        0,
        0
    ),
    LanioModel.LA4T4SP:
    LanioSpec(
        4,
        4,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        0,
        0
    ),
    LanioModel.LAN2R2P:
    LanioSpec(
        2,
        2,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        LanioDeviceType.N_8P,
        0,
        0
    ),
    LanioModel.LA2R3A:
    LanioSpec(
        2,
        3,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        3
    ),
    LanioModel.LA2A3PP:
    LanioSpec(
        0,
        3,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.ANALOG,
        LanioDeviceType.NONE,
        LanioDeviceType.ANALOG,
        LanioDeviceType.ANALOG,
        2,
        0
    ),
    LanioModel.LA2R3AV2:
    LanioSpec(
        2,
        3,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        3
    ),
    LanioModel.LA3A2PP:
    LanioSpec(
        0,
        2,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.ANALOG,
        LanioDeviceType.NONE,
        LanioDeviceType.ANALOG,
        LanioDeviceType.ANALOG,
        3,
        0
    ),
    LanioModel.LA5AI:
    LanioSpec(
        0,
        5,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        LanioDeviceType.NONE,
        0,
        5
    ),
}
